import xbmc
import xbmcaddon
import xbmcgui
import xbmcvfs
import requests
import os

class AddonUpdater:
    def __init__(self):
        self.addon = xbmcaddon.Addon()
        self.ADDON_PATH = xbmcvfs.translatePath(self.addon.getAddonInfo('path'))
        self.VERSION_FILE = os.path.join(self.ADDON_PATH, 'version.txt')
        
        # Leggi le impostazioni dall'addon
        self.REPO_OWNER = "Lucifer4692"
        self.REPO_NAME = "plugin.video.monflix"
        self.GITHUB_TOKEN = "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"
        
        self.UPDATE_FILES = [
            'default.py',
            'mpd_playback.py',
            'hls_playback.py',
            'addon.xml',
            'version.txt'
        ]

    def get_local_version(self):
        """Legge la versione locale dell'addon"""
        try:
            with open(self.VERSION_FILE, 'r') as f:
                return f.read().strip()
        except IOError:
            return "0.0.0"

    def get_file_from_github(self, file_path):
        """Recupera un file da GitHub"""
        url = f'https://api.github.com/repos/{self.REPO_OWNER}/{self.REPO_NAME}/contents/{file_path}'
        
        headers = {
            'Authorization': f'token {self.GITHUB_TOKEN}',
            'Accept': 'application/vnd.github.v3.raw'
        }
        
        try:
            response = requests.get(url, headers=headers)
            response.raise_for_status()
            return response.text
        except requests.exceptions.RequestException as e:
            xbmc.log(f"Errore nel recupero del file {file_path}: {str(e)}", xbmc.LOGERROR)
            return None

    def get_remote_version(self):
        """Ottiene la versione da GitHub"""
        xbmc.log("Recupero versione remota...", xbmc.LOGINFO)
        version_file_content = self.get_file_from_github('version.txt')
        if version_file_content:
            xbmc.log(f"Versione remota trovata: {version_file_content.strip()}", xbmc.LOGINFO)
            return version_file_content.strip()
        else:
            xbmc.log("Impossibile recuperare la versione remota", xbmc.LOGERROR)
            return "0.0.0"

    def update_addon(self):
        """Aggiorna i file dell'addon"""
        try:
            progress = xbmcgui.DialogProgress()
            progress.create("Aggiornamento Addon", "Scaricamento files...")
            
            total_files = len(self.UPDATE_FILES)
            updated_files = []
            
            for index, file_name in enumerate(self.UPDATE_FILES, 1):
                if progress.iscanceled():
                    return False
                
                progress.update(
                    int((index / total_files) * 100),
                    f"Aggiornamento {file_name}..."
                )

                content = self.get_file_from_github(file_name)
                if content is not None:
                    file_path = os.path.join(self.ADDON_PATH, file_name)
                    try:
                        # Backup del file originale
                        if os.path.exists(file_path):
                            backup_path = file_path + '.bak'
                            with open(file_path, 'r', encoding='utf-8') as f:
                                original_content = f.read()
                            with open(backup_path, 'w', encoding='utf-8') as f:
                                f.write(original_content)

                        # Scrivi il nuovo contenuto
                        with open(file_path, 'w', encoding='utf-8') as f:
                            f.write(content)
                        updated_files.append(file_name)
                    except Exception as e:
                        xbmc.log(f"Errore nel salvare il file {file_name}: {str(e)}", xbmc.LOGERROR)
                        # Ripristina il backup se disponibile
                        if os.path.exists(backup_path):
                            with open(backup_path, 'r', encoding='utf-8') as f:
                                backup_content = f.read()
                            with open(file_path, 'w', encoding='utf-8') as f:
                                f.write(backup_content)

            progress.close()
            
            if updated_files:
                xbmc.log(f"Files aggiornati: {', '.join(updated_files)}", xbmc.LOGINFO)
                return True
            return False

        except Exception as e:
            xbmc.log(f"Errore durante l'aggiornamento: {str(e)}", xbmc.LOGERROR)
            return False

    def check_for_updates(self):
        """Controlla e applica gli aggiornamenti se disponibili"""
        try:
            # Verifica che le impostazioni siano state configurate
            if not all([self.REPO_OWNER, self.REPO_NAME, self.GITHUB_TOKEN]):
                xbmc.log("Impostazioni GitHub non configurate. Controlla le impostazioni dell'addon.", xbmc.LOGERROR)
                xbmcgui.Dialog().notification(
                    'Errore Aggiornamento',
                    'Impostazioni GitHub mancanti',
                    xbmcgui.NOTIFICATION_ERROR,
                    3000
                )
                return False

            local_version = self.get_local_version()
            remote_version = self.get_remote_version()

            xbmc.log(f"Versione locale: {local_version}", xbmc.LOGINFO)
            xbmc.log(f"Versione remota: {remote_version}", xbmc.LOGINFO)

            if local_version != remote_version:
                # Chiedi all'utente se vuole aggiornare
                dialog = xbmcgui.Dialog()
                update_choice = dialog.yesno(
                    'Aggiornamento Disponibile',
                    f'È disponibile la versione {remote_version}\n'
                    f'La tua versione attuale è {local_version}\n'
                    'Vuoi aggiornare ora?'
                )
                
                if update_choice:
                    if self.update_addon():
                        xbmcgui.Dialog().notification(
                            'Aggiornamento',
                            'Aggiornamento completato con successo!',
                            xbmcgui.NOTIFICATION_INFO,
                            3000
                        )
                        # Richiedi il riavvio di Kodi
                        dialog = xbmcgui.Dialog()
                        if dialog.yesno('Riavvio Necessario', 'È necessario riavviare Kodi per applicare l\'aggiornamento. Vuoi riavviare ora?'):
                            xbmc.executebuiltin('RestartApp')
                        return True
                    else:
                        xbmcgui.Dialog().notification(
                            'Errore',
                            'Aggiornamento non riuscito',
                            xbmcgui.NOTIFICATION_ERROR,
                            3000
                        )
                else:
                    xbmc.log("Aggiornamento rimandato dall'utente", xbmc.LOGINFO)
            else:
                xbmc.log("Addon già aggiornato all'ultima versione.", xbmc.LOGINFO)
            return False

        except Exception as e:
            xbmc.log(f"Errore nel controllo aggiornamenti: {str(e)}", xbmc.LOGERROR)
            return False
# Crea un'istanza globale dell'updater
updater = AddonUpdater()

# Funzione per il controllo degli aggiornamenti che può essere chiamata da altri file
def check_for_updates():
    return updater.check_for_updates()